/*************************************************************************
         TILE GENERATOR MACRO FILE FOR PERSISTENCE OF VISION 3.1
**************************************************************************

Created by Chris Colefax, 22 May 1999

See "QUICKREF.TXT" for more information

*************************************************************************/

#macro set_tile_camera ()
   #ifndef (tile_size) #declare _TG_size = <1, 1, 1>; #else #declare _TG_size = tile_size*<1, 1, 0> + z; #end
   #ifndef (tile_direction) #declare _TG_dir = <1, 1>; #else #declare _TG_dir = tile_direction*<1, 1, 0>; #end
   #ifndef (tile_offset) #declare tile_offset = 0; #end

   #ifndef (_TG_camera)
      camera {orthographic  right x*_TG_size up y*_TG_size direction z  location <.5, .5, 0>*_TG_size - z*100}
      #declare _TG_camera = true;
   #end
#end

#macro texture_tile (Texture, Seam)
   set_tile_camera ()
   #local TT = texture {Texture scale 1/_TG_size}
   #if (_TG_dir.x) #declare TT = texture {gradient x
      texture_map {[.5-Seam/2 TT] [.5+Seam/2 TT translate x]}}
   #end
   #if (_TG_dir.y) #declare TT = texture {gradient y
      texture_map {[.5-Seam/2 TT] [.5+Seam/2 TT translate y]}}
   #end
   plane {z, 0 hollow texture {TT scale _TG_size} translate -z*tile_offset}
   #declare tile_offset = tile_offset + .1;
#end

#macro pigment_tile (Pigment, Seam)
   texture_tile (texture {pigment {Pigment} finish {ambient 1 diffuse 0}}, Seam)
#end

#macro object_tile (Object)
   set_tile_camera ()
   #local TO = object {Object}
   #if (_TG_dir.x) #declare TO = union {
      object {TO translate -x*_TG_size} object {TO} object {TO translate x*_TG_size}}
   #end
   #if (_TG_dir.y) #declare TO = union {
      object {TO translate -y*_TG_size} object {TO} object {TO translate y*_TG_size}}
   #end
   object {TO translate -z*tile_offset}
#end

#macro random_object_tile (Object, Count, SizeTurb, RotTurb, DepthTurb)
   #ifndef (tile_seed) #local tile_seed = 0; #end
   set_tile_camera ()

   union { #local X = (_TG_dir.x = false ? .5 : 0); #while (X <= 1)
      #local Y = (_TG_dir.y = false ? .5 : 0); #while (Y <= 1)
         #local R1 = seed(tile_seed); #local C = 0; #while (C < Count)
         object {Object
            scale 1 + (rand(R1) - .5) * SizeTurb
            rotate <rand(R1), rand(R1), rand(R1)> * 180 * RotTurb
            translate (<rand(R1), rand(R1), rand(R1)> - .5) * <_TG_size.x, _TG_size.y, DepthTurb>
            translate <X, Y, 0>*_TG_size}
      #declare C = C+1; #end
   #declare Y = Y + 1; #end #declare X = X + 1; #end }
   #undef tile_seed #declare tile_seed = rand(R1)*1e6;
#end

#macro blob_tile (Texture, Count, Size, DepthTurb)
   #ifndef (tile_seed) #local tile_seed = 0; #end
   #ifndef (blob_tile_threshold) #local blob_tile_threshold = .5; #end
   set_tile_camera ()

   blob {threshold blob_tile_threshold
   #local X = (_TG_dir.x = false ? .5 : 0); #while (X <= 1)
      #local Y = (_TG_dir.y = false ? .5 : 0); #while (Y <= 1)
         #local R1 = seed(tile_seed); #local C = 0; #while (C < Count)
         sphere {0, 1, 1
            scale Size*(.5 + rand(R1)*.5)
            translate (<rand(R1), rand(R1), rand(R1)> - .5) * <_TG_size.x, _TG_size.y, DepthTurb>
            texture {Texture} translate <X, Y, 0>*_TG_size}
      #declare C = C+1; #end
   #declare Y = Y + 1; #end #declare X = X + 1; #end
   translate -z*(tile_offset+DepthTurb/2) pigment {rgb 0}}
   #declare tile_offset = tile_offset + DepthTurb;
   #undef tile_seed #declare tile_seed = rand(R1)*1e6;
#end

#macro tile_lights (Count, Seed, MaxRotation)
   #local R1 = seed(Seed); #local CColor = <1, 1, 1>;
   #local C = 0; #while (C < Count)
      light_source {-z*1e6, rgb CColor rotate <rand(R1), rand(R1), rand(R1)>*MaxRotation}
      #declare CColor = .25 + <rand(R1), rand(R1), rand(R1)>*.25;
   #declare C = C + 1; #end
   #declare _TG_lights = true;
#end

#macro random_tile (Seed)
   #local R1 = seed(Seed); #switch (int(rand(R1)*3.99))
   #case (0)
      pigment_tile (random_pigment (rand(R1)*1e6), .2 + rand(R1)*.8)
      #break

   #case (1)
      tile_lights (1 + rand(R1)*3, rand(R1)*1e6, 5 + rand(R1)*90)
      texture_tile (random_texture (rand(R1)*1e6), .2 + rand(R1)*.8)
      #break

   #case (2)
      tile_lights (1 + rand(R1)*3, rand(R1)*1e6, 5 + rand(R1)*90)
      texture_tile (random_wood_texture (rand(R1)*1e6), .2 + rand(R1)*.8)
      #break

   #case (3)
      tile_lights (1 + rand(R1)*3, rand(R1)*1e6, 5 + rand(R1)*25)
      texture_tile (random_texture (rand(R1)*1e6), .2 + rand(R1)*.8)
      #declare tile_seed = rand(R1)*1e6;
      blob_tile (random_texture (rand(R1)*1e6), 5 + rand(R1)*20, .1 + rand(R1)*.9, rand(R1))
   #end
#end

#macro random_pattern (RGen)
   #switch (int(rand(RGen)*15.99))
   #case (0) agate #break
   #case (1) bozo #break
   #case (2) crackle #break
   #case (3) dents #break
   #case (4) granite #break
   #case (5) leopard #break
   #case (6) marble #break
   #case (7) onion #break
   #case (8) quilted #break
   #case (9) radial #break
   #case (10) ripples #break
   #case (11) spiral1 2 #break
   #case (12) spiral2 2 #break
   #case (13) waves #break
   #case (14) wood #break
   #case (15) wrinkles #end
#end

#macro random_mapping (RGen)
   #switch (rand(RGen)*7.99)
   #case (0) ramp_wave #break
   #case (1) triangle_wave #break
   #case (2) sine_wave #break
   #case (3) scallop_wave #break
   #case (4) cubic_wave #break
   #case (5) poly_wave 2 #break
   #case (6) poly_wave 4 #break
   #case (7) poly_wave 8 #end
#end

#macro random_turbulence (RGen)
   turbulence .1+rand(RGen)
   octaves 2 + rand(RGen)*5
   lambda 2+rand(RGen)*4
#end

#macro random_transform (RGen)
   scale .1 + rand(RGen)*3
   rotate (<rand(RGen), rand(RGen), rand(RGen)>-.5)*180
   translate (<rand(RGen), rand(RGen), rand(RGen)>-.5)*100
#end

#macro random_pigment (Seed)
   #local R1 = seed(Seed); pigment { random_pattern(R1)
      color_map { #local C = 0; #while (C < 1.1)
         [C rgb <rand(R1), rand(R1), rand(R1)>]
      #declare C = C + .01 + rand(R1)*.5; #end }
      #if (rand(R1) < .5) random_turbulence (R1) #end
      #if (rand(R1) < .3) random_mapping (R1) #end
      #if (rand(R1) < .2) frequency 1 + rand(R1)*10 #end
      random_transform (R1) }
#end

#macro random_texture (Seed)
   #local R1 = seed(Seed); texture {
   random_pigment (rand(R1)*1e6)
   normal { random_pattern (R1) bump_size -1.5 + rand(R1) * 3
      #if (rand(R1) < .5) random_mapping (R1) #end
      #if (rand(R1) < .2) random_turbulence (R1) #end
      #if (rand(R1) < .2) frequency 1 + rand(R1)*10 #end
      random_transform (R1) }
   finish { 
      #if (rand(R1) < .7) phong .1 + rand(R1)*.9 phong_size 1 + rand(R1)*100 #end
      #if (rand(R1) < .3) reflection .1 + rand(R1)*.7 #end
      #if (rand(R1) < .3) brilliance .5 + rand(R1)*5 #end
      #if (rand(R1) < .3) metallic .5 + rand(R1)*1.5 #end
      #if (rand(R1) < .2) irid {.1 + rand(R1)*.3 thickness rand(R1)*.1} #end
      }}
#end

#macro random_wood_texture (Seed)
   #local R1 = seed(Seed); #local F = 20 + rand(R1)*30; #local T = rand(R1)*.05;
   texture { pigment {wood color_map {
      #local C = 0; #while (C < 2)
         #local CR = .3 + rand(R1)*.5; #local CG = CR*(.4 + rand(R1)*.4); #local CB = CG*(.2 + rand(R1)*.6);
         [C rgb <CR, CG, CB>]
      #declare C = C + 1; #end }
      random_mapping (R1) frequency F turbulence T}
   #if (rand(R1) < .33)
      normal {wood random_mapping (R1) bump_size -.3 + rand(R1)*.6 frequency F turbulence T}
   #end
   #if (rand(R1) < .25)
      finish {phong .1 + rand(R1)*.2 phong_size 10 + rand(R1)*30 }
   #end
   rotate <rand(R1), rand(R1), rand(R1)>*180
   translate (<rand(R1), rand(R1), rand(R1)>-.5)*10}
#end
